/*==============================================================================*/
/*  Copyright (C) 2020, Heartland Data inc. All Rights Reserved.                */
/*                                                                              */
/*  Title  :   Data Writing Driver                                              */
/*  EventID:   Insert Extended EventID Test Point                               */
/*  AppVer :   DT+ Ver1.00`                                                    */
/*  FileID :   16a                                                              */
/*  Version:   1.0                                                              */
/*  Author :   HLDC                                                             */
/*==============================================================================*/

/*==============================================================================*/
/*  Desc:   Header for File IO                                                  */
/*==============================================================================*/
#include <stdio.h>

/*==============================================================================*/
/*  Macro:  DT_UINT                                                             */
/*  Desc:   Please change Test Point argument type for DT+ Project setting.     */
/*==============================================================================*/
#define	DT_UINT unsigned int

/*==============================================================================*/
/*  Desc:   Test Point Parametaer                                               */
/*==============================================================================*/
#define	DT_VARIABLE_BIT		0x2
#define	DT_EVTTRG_BIT		0x8

static FILE *dat_file = NULL;
static int BusoutMode = 0;
static unsigned int time = 0;

/*==============================================================================*/
/*  Macro:  BusOutModeSet                                                       */
/*  Desc:   0 => open file 1 => writing 2 => close file                         */
/*==============================================================================*/
#define BusOutModeSet(mode) BusoutMode = mode

/*==============================================================================*/
/*  Func:   _TP_BusOut                                                          */
/*  Desc:   Called by Test Point                                                */
/*==============================================================================*/
void _TP_BusOut( DT_UINT addr, DT_UINT dat )
{
	unsigned char buff[16] = {0};

	if (BusoutMode == 0) {
		/* Customize File Path */
		dat_file = fopen("ConvertData.DAT", "wb+");
		if (dat_file == NULL) {
			return;
		}
		BusOutModeSet(1);
	}

	/* time data(Customize) */
	time++;

	buff[0] = (unsigned char)((dat) & 0xFF);
	buff[1] = (unsigned char)((dat >> 8) & 0xFF);
	buff[2] = (unsigned char)((addr) & 0xFF);
	buff[3] = (unsigned char)((addr >> 8) & 0xFF);
	buff[4] = (unsigned char)((addr >> 16) & 0xFF);

	/* time data */
	buff[5] = (unsigned char)((time) & 0xFF);
	buff[6] = (unsigned char)((time >> 8) & 0xFF);
	buff[7] = (unsigned char)((time >> 16) & 0xFF);
	buff[8] = (unsigned char)((time >> 24) & 0xFF);

	/* Reserve */
	buff[9] = 0x00;
   	buff[10] = 0x00;
   	buff[11] = 0x00;
   	buff[12] = 0x00;
	buff[13] = 0x00;
   	buff[14] = 0x00;
   	buff[15] = 0x00;

	fwrite(buff, sizeof(char), 16, dat_file);
	if (BusoutMode == 2) {
		fclose(dat_file);
	}
}

/*==============================================================================*/
/*  Func:   _TP_MemoryOutput                                                    */
/*  Desc:   Called by Variable TestPoint                                        */
/*==============================================================================*/
void _TP_MemoryOutput( DT_UINT addr, DT_UINT dat, void *value, DT_UINT size )
{
	char buff[16];
	char *p = (char*)value;
	int i, j, packetNum, count;

	if (size > 256) size = 256;
	packetNum = (size + 7) / 8;

	_TP_BusOut((addr | DT_VARIABLE_BIT), dat);	/* Memory Dump Start */

	buff[0] = size;
	buff[1] = packetNum;
	buff[2] = 0x03;
	buff[12] = 0x00;
	buff[13] = 0x00;
	buff[14] = 0x00;
	buff[15] = 0x00;

	count = 0;
	for (i = 0; i < packetNum; i++) {
		buff[3] = i;
		for (j = 0; j < 8; j++, count++, p++) {
			buff[4+j] = (count < size)?*p:0x00;
		}
		fwrite(buff, sizeof(char), 16, dat_file);
	}
	if (BusoutMode == 2) {
		fclose(dat_file);
	}
}

/*==============================================================================*/
/*  Func:   _TP_EventTrigger32                                                  */
/*  Desc:   Called by Event Trigger Test Point(32bit mode)                      */
/*==============================================================================*/
void _TP_EventTrigger32( DT_UINT addr, DT_UINT dat, DT_UINT event_id )
{
	unsigned char buff[16] = {0};

	if (BusoutMode == 0) {
		/* Customize File Path */
		dat_file = fopen("ConvertData.DAT", "wb+");
		if (dat_file == NULL) {
			return;
		}
		BusOutModeSet(1);
	}

	/* time data(Customize) */
	time++;

	addr |= DT_EVTTRG_BIT;
	buff[0] = (unsigned char)((dat) & 0xFF);
	buff[1] = (unsigned char)((dat >> 8) & 0xFF);
	buff[2] = (unsigned char)((addr) & 0xFF);
	buff[3] = (unsigned char)((addr >> 8) & 0xFF);
	buff[4] = (unsigned char)((addr >> 16) & 0xFF);

	/* time data */
	buff[5] = (unsigned char)((time) & 0xFF);
	buff[6] = (unsigned char)((time >> 8) & 0xFF);
	buff[7] = (unsigned char)((time >> 16) & 0xFF);
	buff[8] = (unsigned char)((time >> 24) & 0xFF);

	/* event_id */
	buff[9] = (unsigned char)((event_id) & 0xFF);
	buff[10] = (unsigned char)((event_id >> 8) & 0xFF);
	buff[11] = (unsigned char)((event_id >> 16) & 0xFF);
	buff[12] = (unsigned char)((event_id >> 24) & 0xFF);

	/* Reserve */
	buff[13] = 0x00;
   	buff[14] = 0x00;
   	buff[15] = 0x00;

	fwrite(buff, sizeof(char), 16, dat_file);
	if (BusoutMode == 2) {
		fclose(dat_file);
	}
}
